﻿namespace ProjectCleaningCar.CollectionGenericObjects;
/// <summary>
/// Параметризованный набор объектов
/// </summary>
/// <typeparam name="T">Параметр: ограничение - ссылочный тип</typeparam>
public class MassiveGenericObjects<T> : ICollectionGenericObjects<T>
    where T : class
{
    /// <summary>
    /// Массив объектов, которые храним
    /// </summary>
    private T?[] _collection;
    public int Count => _collection.Length;
    public int MaxCount
    {
        get
        {
            return _collection.Length;
        }
        set
        {
            if (value > 0)
            {
                if (_collection.Length > 0)
                {
                    Array.Resize(ref _collection, value);
                }
                else
                {
                    _collection = new T?[value];
                }
            }
        }
    }

    public CollectionType GetCollectionType => CollectionType.Massive;
    /// <summary>
    /// Конструктор
    /// </summary>
    public MassiveGenericObjects()
    {
        _collection = Array.Empty<T?>();
    }
    public T? Get(int position)
    {
        if (position < 0 || position >= Count) return null;
        return _collection[position];
    }
    public int Insert(T obj)
    {
        for (int i = 0; i < Count; i++)
        {
            if (_collection[i] == null)
            {
                _collection[i] = obj;
                return i;
            }
        }
        return -1;
    }
    public int Insert(T obj, int position)
    {
        if (position >= Count || position < 0) return -1;
        if (_collection[position] == null)
        {
            _collection[position] = obj;
            return position;
        }
        int temp = position + 1;
        while(temp < Count)
        {
            if (_collection[temp] == null)
            {
                _collection[temp] = obj;
                return temp;
            }
            ++temp;
        }
        temp = position - 1;
        while(temp >= 0)
        {
            if (_collection[temp] == null)
            {
                _collection[temp] = obj;
                return temp;
            }
            --temp;
        }
        return -1;
    }
    public T? Remove(int position)
    {
        if (position >= Count || position < 0) return null;
        T? myObject = _collection[position];
        _collection[position] = null;
        return myObject;
    }

    public IEnumerable<T?> GetItems()
    {
        for (int i = 0; i < _collection.Length; ++i)
        {
            yield return _collection[i];
        }
    }
}